////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : Inject/Eject Library Demo
//  * Unit Name : main
//  * Purpose   :      CreateRemoteThread
//  * Author    :  (Rouse_) 
//  * Version   : 1.00
//  ****************************************************************************
//

unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

resourcestring
  BTN_INJECT = 'Inject';
  BTN_EJECT  = 'Eject';
  
const
  DLLName = 'hooklib.dll';

type
  TfrmMain = class(TForm)
    btnInjectEject: TButton;
    GroupBox: TGroupBox;
    lbStatus: TListBox;
    procedure btnInjectEjectClick(Sender: TObject);
  private
    function InjectLib(const ProcessID: DWORD): Boolean;
    function EjectLib(const ProcessID: DWORD): Boolean;
  end;

  //         
  TGetModuleHandle = function (lpModuleName: PChar): HMODULE; stdcall;
  TFreeLibrary = function (hLibModule: HMODULE): BOOL; stdcall;

  //       
  PEjectLibStruct = ^TEjectLibStruct;
  TEjectLibStruct = record
    hGetModuleHandle: TGetModuleHandle;
    hFreeLibrary: TFreeLibrary;
    lpModuleName: PChar;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

{ TfrmMain }

//     / 
// =============================================================================
procedure TfrmMain.btnInjectEjectClick(Sender: TObject);
begin
  TComponent(Sender).Tag := TComponent(Sender).Tag + 1;
  if (TComponent(Sender).Tag mod 2) = 1 then
  begin
    btnInjectEject.Caption := BTN_EJECT;
    if InjectLib(GetCurrentProcessID) then
      lbStatus.Items.Add('Library injected succes.')
    else
      lbStatus.Items.Add('Library injected fail.');
  end
  else
  begin
    btnInjectEject.Caption := BTN_INJECT;
    if EjectLib(GetCurrentProcessID) then
      lbStatus.Items.Add('Library ejected succes.')
    else
      lbStatus.Items.Add('Library ejected fail.');
  end;
end;

//        ,
//       
// =============================================================================
function SetDebugPriv: Boolean;
var
  Token: THandle;
  tkp: TTokenPrivileges;
  ReturnLength: DWORD;
begin
  Result := false;
  //    
  if OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, Token) then
  begin
    //  Luid 
    if LookupPrivilegeValue(nil, PChar('SeDebugPrivilege'), tkp.Privileges[0].Luid) then
    begin
      //   
      tkp.PrivilegeCount := 1;
      tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
      //  
      Result := AdjustTokenPrivileges(Token, false, tkp, 0, nil, ReturnLength);
    end;
  end;
end;     

//         PID  ProcessID
//         LoadLibraryA
//      .
//           
// =============================================================================
function TfrmMain.InjectLib(const ProcessID: DWORD): Boolean;
var
  Process: HWND;
  ThreadRtn: FARPROC;
  DllPath: String;
  RemoteDll: Pointer;
  BytesWriten: DWORD;
  Thread: DWORD;
  ThreadId: DWORD;
  ExitCode: DWORD;
begin
  //      
  Result := SetDebugPriv;
  if not Result then Exit;
  Process := 0;
  Thread := 0;
  try
    //  
    Process := OpenProcess(PROCESS_CREATE_THREAD or PROCESS_VM_OPERATION or
      PROCESS_VM_WRITE, True, ProcessID);
    if Process = 0 then Exit;
    //      
    DllPath := ExtractFilePath(ParamStr(0)) + DLLName;
    RemoteDll := VirtualAllocEx(Process, nil, Length(DllPath),
      MEM_COMMIT or MEM_TOP_DOWN, PAGE_READWRITE);
    if RemoteDll = nil then Exit;
    //        
    if not WriteProcessMemory(Process, RemoteDll, PChar(DllPath),
      Length(DllPath), BytesWriten) then Exit;
    if BytesWriten <> DWORD(Length(DllPath)) then Exit;
    //     Kernel32.dll
    ThreadRtn := GetProcAddress(GetModuleHandle('Kernel32.dll'), 'LoadLibraryA');
    if ThreadRtn = nil then Exit;
    //   
    Thread := CreateRemoteThread(Process, nil, 0, ThreadRtn, RemoteDll, 0, ThreadId);
    if Thread = 0 then Exit;
    //     ...
    if (WaitForSingleObject(Thread, INFINITE) = WAIT_OBJECT_0) then
      if GetExitCodeThread(Thread, ExitCode) then
        Result := ExitCode = 0;
  finally
    //         ,
    //    ...
    if RemoteDll <> nil then
      VirtualFreeEx(Process, @RemoteDll, 0, MEM_RELEASE);
    if Thread <> 0 then CloseHandle(Thread);
    if Process <> 0 then CloseHandle(Process);  
  end;
end;

//      ,      
//       FreeLibrary
//         
//        3 .
//  1:   GetModuleHandle  FreeLibrary;
//  2:  ,     
// =============================================================================
function RemoteFreeLibrary(lpParameter: Pointer): DWORD; stdcall;
var
  hLibModule: HMODULE;
begin
  Result := 0;
  if lpParameter = nil then Exit;
  //     (  )
  hLibModule := TGetModuleHandle(PEjectLibStruct(lpParameter)^.hGetModuleHandle)
    (PEjectLibStruct(lpParameter)^.lpModuleName);
  if hLibModule <> 0 then
    //  
    Result := DWORD(TFreeLibrary(PEjectLibStruct(lpParameter)^.hFreeLibrary)(hLibModule));
end;

//        
//     RemoteFreeLibrary
//        
// =============================================================================
function TfrmMain.EjectLib(const ProcessID: DWORD): Boolean;
var
  Process: HWND;
  BytesWriten: DWORD;
  Thread: DWORD;
  ThreadId: DWORD;
  ExitCode: DWORD;
  EjectLibStruct: TEjectLibStruct;
  EjectLibStructAddr: Pointer;
begin
  Result := False;
  Process := 0;
  Thread := 0;
  try
    //  
    Process := OpenProcess(PROCESS_CREATE_THREAD or PROCESS_VM_OPERATION or
      PROCESS_VM_WRITE, True, ProcessID);
    if Process = 0 then Exit;
    //       
    EjectLibStruct.lpModuleName := VirtualAllocEx(Process, nil, Length(DLLName),
      MEM_COMMIT or MEM_TOP_DOWN, PAGE_READWRITE);
    if EjectLibStruct.lpModuleName = nil then Exit;
    //       
    if not WriteProcessMemory(Process, EjectLibStruct.lpModuleName, PChar(DLLName),
      Length(DLLName), BytesWriten) then Exit;
    if BytesWriten <> DWORD(Length(DLLName)) then Exit;
    //    FreeLibrary  Kernel32.dll
    EjectLibStruct.hFreeLibrary :=
      GetProcAddress(GetModuleHandle('Kernel32.dll'), 'FreeLibrary');
    if not Assigned(EjectLibStruct.hFreeLibrary) then Exit;
    //    GetModuleHandle  Kernel32.dll
    EjectLibStruct.hGetModuleHandle :=
      GetProcAddress(GetModuleHandle('Kernel32.dll'), 'GetModuleHandleA');
    if not Assigned(EjectLibStruct.hGetModuleHandle) then Exit;
    //    ,    
    EjectLibStructAddr := VirtualAllocEx(Process, nil, SizeOf(TEjectLibStruct),
      MEM_COMMIT or MEM_TOP_DOWN, PAGE_READWRITE);
    if EjectLibStructAddr = nil then Exit;
    //   
    if not WriteProcessMemory(Process, EjectLibStructAddr, @EjectLibStruct,
      SizeOf(TEjectLibStruct), BytesWriten) then Exit;
    if BytesWriten <> DWORD(SizeOf(TEjectLibStruct)) then Exit;

    //   
    Thread := CreateRemoteThread(Process, nil, 0, @RemoteFreeLibrary,
      EjectLibStructAddr, 0, ThreadId);
    if Thread = 0 then Exit;
    //     ...
    if (WaitForSingleObject(Thread, INFINITE) = WAIT_OBJECT_0) then
      if GetExitCodeThread(Thread, ExitCode) then
        Result := BOOL(ExitCode);
  finally
    //         ,
    //    ...
    VirtualFreeEx(Process, @EjectLibStruct.lpModuleName, 0, MEM_RELEASE);
    VirtualFreeEx(Process, @EjectLibStructAddr, 0, MEM_RELEASE);
    if Thread <> 0 then CloseHandle(Thread);
    if Process <> 0 then CloseHandle(Process);
  end;
end;

end.
